//
// Fragment shader for Gooch shading
//
// Author: Randi Rost
//
// Copyright (c) 2002-2006 3Dlabs Inc. Ltd.
//
// See 3Dlabs-License.txt for license information
//

// Modified by A. Julian Mayer in 2008 for Texturing & Shadowmapping

uniform vec3  SurfaceColor; // (0.75, 0.75, 0.75)
uniform vec3  WarmColor;    // (0.6, 0.6, 0.0)
uniform vec3  CoolColor;    // (0.0, 0.0, 0.6)
uniform float DiffuseWarm;  // 0.45
uniform float DiffuseCool;  // 0.45
uniform sampler2D texUnit0;

varying float NdotL;
#ifndef NO_SPECULAR	
varying vec3  ReflectVec;
varying vec3  ViewVec;
#endif
#ifdef QUAKE_DISRUPTOR
varying vec4 qdAffected;
#endif

uniform sampler2DShadow shadowMap[3];
varying vec4 shadowTexCoord[3];

float lookup(vec2 xy)
{
    int i;
    float depth;

    //for(i = 0; i < shadowCount; i++)		
    //depth += shadow2DProj(shadowMap[i], shadowTexCoord[i] + vec4(xy.x, xy.y, 0, 0) * 0.1).x; // BROKEN LOOP AGAIN; WHY OH WHY;

	depth += shadow2DProj(shadowMap[0], shadowTexCoord[0] + vec4(xy.x, xy.y, 0, 0) * 0.01).x;
    depth += shadow2DProj(shadowMap[1], shadowTexCoord[1] + vec4(xy.x, xy.y, 0, 0) * 0.01).x;
	depth += shadow2DProj(shadowMap[2], shadowTexCoord[2] + vec4(xy.x, xy.y, 0, 0) * 0.01).x;

    return (depth != 3.0) ? 0.5 : 1.0;
}

void main (void)
{
    vec3 kcool    = min(CoolColor + DiffuseCool * SurfaceColor, 1.0);
    vec3 kwarm    = min(WarmColor + DiffuseWarm * SurfaceColor, 1.0); 
    vec3 kfinal   = mix(kcool, kwarm, NdotL);

#ifdef NO_SPECULAR	
    vec4 color = vec4 (min(kfinal, 1.0), 1.0);
#else
    vec3 nreflect = normalize(ReflectVec);
    vec3 nview    = normalize(ViewVec);

    float spec    = max(dot(nreflect, nview), 0.0);
    spec          = pow(spec, 32.0);

    vec4 color = vec4 (min(kfinal + spec, 1.0), 1.0);
#endif
	
#ifdef TEXTURING
	color *= texture2D(texUnit0, gl_TexCoord[0].xy);
#endif

//	vec2 o = mod(floor(gl_FragCoord.xy), 2.0);
//	float sum = 0.0;										
//															
//	sum += lookup(vec2(-1.5, 1.5) + o);					
//	sum += lookup(vec2( 0.5, 1.5) + o);					
//	sum += lookup(vec2(-1.5, -0.5) + o);					
//	sum += lookup(vec2( 0.5, -0.5) + o);					
//	gl_FragColor = vec4(sum * 0.25 * color.rgb, color.a); 

//	float i, v;												
//	for (i = -1.5; i <= 1.5; i+=1.0)					
//		for (v = -1.5; v <= 1.5; v+=1.0)					
//			sum += lookup(vec2(i, v));						
//																
//	gl_FragColor = vec4(sum * 0.0625 * color.rgb, color.a);

	gl_FragColor = vec4(lookup(vec2(0.0,0.0)) * color.rgb, color.a);
	
	#ifdef QUAKE_DISRUPTOR
		gl_FragColor -= qdAffected;
	#endif
}