//
// Vertex shader for Gooch shading
//
// Author: Randi Rost
//
// Copyright (c) 2002-2006 3Dlabs Inc. Ltd.
//
// See 3Dlabs-License.txt for license information
//

// Modified by A. Julian Mayer in 2008 for Texturing, Shadowmapping and NoSpecular

uniform mat4 sm[3];
varying vec4 shadowTexCoord[3];

uniform vec3  LightPosition;  // (0.0, 10.0, 4.0) 

#ifdef QUAKE_DISRUPTOR
uniform vec4 qdPosition;
uniform bool qdEnabled;
varying vec4 qdAffected;
#endif

varying float NdotL;
#ifndef NO_SPECULAR		
varying vec3  ReflectVec;
varying vec3  ViewVec;
#endif

void main(void)
{
    vec3 ecPos      = vec3 (gl_ModelViewMatrix * gl_Vertex);
    vec3 tnorm      = normalize(gl_NormalMatrix * gl_Normal);
    vec3 lightVec   = normalize(LightPosition - ecPos);
#ifndef NO_SPECULAR		
    ReflectVec      = normalize(reflect(-lightVec, tnorm));
    ViewVec         = normalize(-ecPos);
#endif
    NdotL           = (dot(lightVec, tnorm) + 1.0) * 0.5;
	
#ifdef TEXTURING	
    gl_TexCoord[0] = gl_MultiTexCoord0;	
#endif
    gl_Position     = ftransform();
	
	#ifdef QUAKE_DISRUPTOR
	qdAffected = vec4(0.0, 0.0, 0.0, 0.0);	
	if (qdEnabled)
	{
		float dis = distance(vec3(gl_Vertex), vec3(qdPosition));
		if (dis < 40.0)
		{
			gl_Position.y += 10.0;
			qdAffected = vec4(1.0, 0.0, 0.0, 0.0);
		}
	}
	#endif
	
    shadowTexCoord[0] = sm[0] * gl_ModelViewMatrix * gl_Vertex;
    shadowTexCoord[1] = sm[1] * gl_ModelViewMatrix * gl_Vertex;	
    shadowTexCoord[2] = sm[2] * gl_ModelViewMatrix * gl_Vertex;	
}