uniform sampler2DRect colorTexture;

#ifdef LS
uniform vec3 screenLightPos;
uniform float decay;
uniform float weight;
uniform float exposure;

const float NUM_SAMPLES = 64.0;
#else
uniform sampler2DRect depthTexture;
uniform int numSamples;
uniform int g_width;
uniform int g_height;
uniform mat4 g_ViewProjectionInverseMatrix;
uniform mat4 g_previousViewProjectionMatrix;
#endif

void main()
{
#ifdef LS
		vec2 texCoord = gl_TexCoord[0].st;
        vec2 deltaTexCoord = (texCoord.st - screenLightPos.xy) / NUM_SAMPLES;   //screen direction from pixel to light
        if (screenLightPos.z < 0.0) 
			deltaTexCoord = -deltaTexCoord;                //to handle light in the back

        vec3 color = texture2DRect(colorTexture, gl_TexCoord[0].st).rgb;           //first sample

        float illuminationDecay = 1.0;

        for(float i=0.0;i<NUM_SAMPLES;i++)                                      //the main loop
        {
                texCoord -= deltaTexCoord;
                vec3 sample = texture2DRect(colorTexture, texCoord.st).rgb;

                sample *= illuminationDecay * weight;
                color += sample;
                illuminationDecay *= decay;
        }

        gl_FragColor = vec4(color*exposure,1.0);
#else
	vec2 texCoord;
	float zOverW;
	vec4 tmp1;
	vec4 tmp2;
	vec4 H;
	vec4 worldPos;
	vec4 previousPos;
	vec2 velocity;
	vec4 color;
	int i;

	// Shader Code That Extracts the Per-Pixel World-Space Positions of the Objects That Were Rendered to the Depth Buffer
	// Get the depth buffer value at this pixel.
	zOverW = texture2DRect(depthTexture, gl_TexCoord[0].xy).x;

	// H is the viewport position at this pixel in the range -1 to 1.
	H = vec4((gl_TexCoord[0].x/float(g_width))*2.0 - 1.0, (1.0 - (gl_TexCoord[0].y/float(g_height)))*2.0 - 1.0, zOverW, 1.0);

	// Transform by the view-projection inverse.
	tmp1 = H * g_ViewProjectionInverseMatrix;
	// Divide by w to get world position.
	worldPos = tmp1/tmp1.w;

	// Shader Code That Computes the Per-Pixel Velocity Vectors That Determine the Direction to Blur the Image
	// Use the world position, and transform by the previous view-projection matrix.
	tmp2 = worldPos * g_previousViewProjectionMatrix;
	// Convert to nonhomogenous points [-1,1] by dividing by w.
	previousPos = tmp2/tmp2.w;

	// Use this frame's position and last frame's to compute the pixel velocity.
	velocity = ((H - previousPos)/4.0).xy;
	velocity.x *= float(g_width)/float(numSamples);
	velocity.y *= float(g_height)/float(numSamples);

	// Shader Code That Uses the Velocity Vector at the Current Pixel to Sample the Color Buffer Multiple Times to Achieve the Motion Blur Effect
	// Get the initial color at this pixel.
	color = texture2DRect(colorTexture, gl_TexCoord[0].xy);

	texCoord = gl_TexCoord[0].xy + velocity;
 	 for (i = 1; i < numSamples; i++, texCoord += velocity) 
	{
		// Sample the color buffer along the velocity vector.
		vec4 currentColor = texture2DRect(colorTexture, texCoord);

		// Add the current color to our color sum.
		color += currentColor;
	}

	// Average all of the samples to get the final blur color.
	gl_FragColor = color/float(numSamples+1);
	gl_FragDepth = zOverW;
#endif
}
